@@ -14,6 +14,7 @@ from simple_http_server import Parameter
 from simple_http_server import MultipartFile
 from simple_http_server import Response
 from simple_http_server import Request
+from simple_http_server import Session
 from simple_http_server import filter_map
 from simple_http_server import request_map
 import os
@@ -36,7 +37,7 @@ def my_ctrl():
 @request_map("/say_hello", method=["GET", "POST"])
 def my_ctrl2(name, name2=Parameter("name", default="KEIJACK")):
     """name and name2 is the same"""
-    return "<!DOCTYPE html><html><body>hello, %s, %s</body></html>" % (name, name2)
+    return f"<!DOCTYPE html><html><body>hello, {name}, {name2}</body></html>"
 
 
 @request_map("/error")
@@ -57,20 +58,20 @@ def my_upload(img=MultipartFile("img"), txt=Parameter("中文text", required=Fal
 
     root = os.path.dirname(os.path.abspath(__file__))
     img.save_to_file(root + "/imgs/" + img.filename)
-    return "<!DOCTYPE html><html><body>upload ok! %s </body></html>" % txt
+    return f"<!DOCTYPE html><html><body>upload ok! {txt} </body></html>"
 
 
 @request_map("/post_txt", method=["GET", "POST"])
 def normal_form_post(txt=Parameter("中文txt", required=False, default="DEFAULT"), req=Request()):
     for k, v in req.parameter.items():
         print("%s ====> %s " % (k, v))
-    return "<!DOCTYPE html><html><body>hi, %s</body></html>" % txt
+    return f"<!DOCTYPE html><html><body>hi, {txt}</body></html>"
 
 
 @request_map("/upload", method="GET")
 def show_upload():
     root = os.path.dirname(os.path.abspath(__file__))
-    return StaticFile("%s/my_test_index.html" % root, "text/html; charset=utf-8")
+    return StaticFile(f"{root}/my_test_index.html", "text/html; charset=utf-8")
 
 
 @request_map("/a.mov", method="GET")
@@ -85,7 +86,7 @@ def post_json(json=JSONBody()):
 
 
 @request_map("/headers")
-def set_headers(res=Response(), headers=Headers(), cookies=Cookies(), cookie=Cookie("sc")):
+def set_headers(res: Response, headers: Headers, cookies: Cookies, cookie=Cookie("sc")):
     print("==================cookies==========")
     print(cookies)
     print("==================cookies==========")
@@ -100,6 +101,19 @@ def tuple_results():
     return 200, Headers({"my-header": "headers"}), {"success": "成功！"}
 
 
+@request_map("session")
+def test_session(session: Session, invalid=False):
+    ins = session.get_attribute("in-session")
+    if not ins:
+        session.set_attribute("in-session", "Hello, Session!")
+
+    __logger.info("session id: %s" % session.id)
+    if invalid:
+        __logger.info("session[%s] is being invalidated. " % session.id)
+        session.invalidate()
+    return "<!DOCTYPE html><html><body>%s</body></html>" % str(ins)
+
+
 @request_map("tuple_cookie")
 def tuple_with_cookies(headers=Headers(), all_cookies=Cookies(), cookie_sc=Cookie("sc")):
     print("=====>headers")
@@ -147,6 +161,7 @@ def stop():
     server.stop()
     return {"stopped": True}
 
+
 @request_map("/params")
 def my_ctrl4(user_name,
              password=Parameter(name="passwd", required=True),
@@ -180,3 +195,17 @@ def return_int(status_code=200):
 def my_path_val_ctr(val=PathValue("pval"), path_val=PathValue()):
     __logger.info("val is %s , path_val is %s" % (val, path_val))
     return "<html><body>%s, %s</body></html>" % (val, path_val)
+
+
+class MyController:
+
+    def __init__(self) -> None:
+        self._name = "ctr object"
+
+    def my_ctrl_mth(self, name: str):
+        return {"message": f"hello, {name}, {self._name} says. "}
+
+
+my_ctrl_obj = MyController()
+
+request_map("/obj/say_hello")(my_ctrl_obj.my_ctrl_mth)
